import re
import logging
import importlib

from .source import get_source
from .scheme import get_scheme
from .backend import get_backends
from .exceptions import RegistrationError


logger = logging.getLogger(__name__)


def register_file(config: dict, path: str, replace: bool=False):
    """ Handle the registration of a single path.
    """
    logger.info(f"Handling '{path}'.")
    source = get_source(config, path)
    scheme = get_scheme(config, path)
    context = scheme.get_context(source, path)

    for pre_handler in get_pre_handlers(config):
        pre_handler(config, path, context)

    for backend in get_backends(config, path):
        if backend.exists(source, context):
            if replace:
                logger.info(f"Replacing '{path}'.")
                backend.register(source, context, replace=True)
            else:
                raise RegistrationError(f'Object {context} is already registered')
        else:
            logger.info(f"Registering '{path}'.")
            backend.register(source, context, replace=False)

    for post_handler in get_post_handlers(config):
        post_handler(config, path, context)

    logger.info(f"Successfully {'replaced' if replace else 'registered'} '{path}'")
    return context


def _get_handlers(config, name):
    handlers = []
    for handler_def in config.get(name, []):
        module_path, handler_name = handler_def['path'].rpartition('.')
        handler_cls = getattr(importlib.import_module(module_path), handler_name)
        handlers.append(
            handler_cls(
                *handler_def.get('args', []),
                **handler_def.get('kwargs', []),
            )
        )

    return handlers


def get_pre_handlers(config):
    return _get_handlers(config, 'pre_handlers')


def get_post_handlers(config):
    return _get_handlers(config, 'post_handlers')
